# -*- coding: utf-8 -*-

"""Top-level package for favico."""

from favico.favico import generate_from_color, generate_from_image

__author__ = """romnn"""
__email__ = "contact@romnn.com"
__version__ = "0.1.6"

__all__ = ["generate_from_image", "generate_from_color"]
